/*
 * DiscreteValidator_impl.hpp
 *
 *  Created on: Apr 16, 2012
 *      Author: ankele
 */

#ifndef DISCRETEVALIDATOR_IMPL_HPP_
#define DISCRETEVALIDATOR_IMPL_HPP_


// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Parameters/ParameterExceptions.hpp"
#include <algorithm>

template <class T>
bool DiscreteValidator<T>::isValid(const T & _value) const
{
  typename std::vector<T>::const_iterator iter = std::find(ValidValues.begin(), ValidValues.end(), _value);
  if (iter != ValidValues.end()) {
    //std::cout << "Found " << _value << ":" << *iter << std::endl;
    return true;
  } else {
    //std::cout << "Did not find " << _value << "." << std::endl;
    return false;
  }
}

template <class T>
Validator<T>* DiscreteValidator<T>::clone() const
{
  return new DiscreteValidator<T>(ValidValues);
};


template <class T>
bool DiscreteValidator<T>::operator==(const Validator<T> &_instance) const
{
  const DiscreteValidator<T> *inst = dynamic_cast<const DiscreteValidator<T> *>(&_instance);
  if (inst)
    return ValidValues == inst->ValidValues;
  return false;
};

template <class T>
void DiscreteValidator<T>::appendValidValue(const T &_value) throw(ParameterValidatorException)
{
  if (isValid(_value))
    throw ParameterValidatorException();
  ValidValues.push_back(_value);
}


template <class T>
const std::vector<T> &DiscreteValidator<T>::getValidValues() const
{
  return ValidValues;
}

/** Internal function for finding the index of a desired value.
 *
 * \note As this is internal, we do not ASSERT value's presence, but return -1
 * such that other functions may ASSERT on that.
 *
 * \param _value value to get the index of
 * \return index such that ValidValues[index] == _value
 */
template <class T>
const size_t DiscreteValidator<T>::findIndexOfValue(const T &_value) const
{
  size_t index = 0;
  const size_t max = ValidValues.size();
  for (; index < max; ++index) {
    if (ValidValues[index] == _value)
      break;
  }
  if (index == max)
    return (size_t)-1;
  else
    return index;
}


#endif /* DISCRETEVALIDATOR_IMPL_HPP_ */
